/**
 * Line.js
 * @author  Tom Legrady
 * @date    Dec. 24,  2011
 */
function Line( midpoint, theta, len) {
    "use strict";
    if ( ! midpoint ) {         // Subclass calls constructor with no args for
        return this;            // prototype chaining ... return a null object.
    }
    this.mid = midpoint;
    this.theta = canonical_angle(theta);
    this.halfLen = len / 2;

    var dx = Math.round( this.halfLen * Math.cos( theta )),
	dy = Math.round( this.halfLen * Math.sin( theta ));

    this.p1 = new Point( midpoint.x + dx, midpoint.y + dy );
    this.p2 = new Point( midpoint.x - dx, midpoint.y - dy );

    if ( this.p1.x > this.p2.x
         || ( this.isVertical() && this.p1.y > this.p2.y)
       ) {
        var tmp = this.p1;
        this.p1 = this.p2;
        this.p2 = tmp;
    }
    var dx = this.p2.x - this.p1.x;
    var dy = this.p2.y - this.p1.y;
    this.m = Math.abs( dx ) < epsilon() ? infinity() *infinity() : dy / dx;
    this.b = this.p1.y - this.m * this.p1.x;
}

Line.method( 'distance', function( p ){
    "use strict";
    // Handle vertical line
    if ( this.isVertical() ) {
        return this.distanceFromVerticalLine( p );
    }
    return this.distanceFromNonVerticalLine( p );
});

Line.method( 'isVertical', function() {
    "use strict";
    return Math.abs( this.p1.x - this.p2.x ) <  epsilon() ;
});

Line.method( 'distanceFromVerticalLine', function( p ) {
    "use strict";
    // not on line
    if ( Math.abs( this.p1.x - p.x ) >  epsilon() ) {
        return Math.abs( this.p1.x - p.x );
    }
    // on line, not within line segment
    if ( p.y < this.p1.y || p.y > this.p2.y ) {
        return Math.min( Math.abs(p.y - this.p1.y), Math.abs(this.p2.y - p.y) );
    }
    // on line segment.
    return 0;
});

Line.method( 'distanceFromNonVerticalLine', function( p ) {
    "use strict";
    // A line is defined by : y = mx + b or bringing y to other side
    // that becomes mx - y + b = 0. Renaming the constants to become
    // ax +by + c = 0, the distance fo a point (m, n) is given by:
    // abs( am + bn + c ) / sqrt( a^2 + b^2 )
    // 
    var num = Math.abs( this.m * p.x - p.y + this.b ),
      denom = Math.sqrt( this.m * this.m + 1 );
    if ( num ) {                // point is to side of line
        return  num / denom;
    }
    if ( this.pointInBounds( p ) ) { // point falls on line segment
        return 0;
    }
    // point falls on line, but outside segment
    var dp1 = this.p2.distance( p ),
    dp2 = this.p2.distance( p );
    
    return Math.min( dp1, dp2 );

});
Line.method( 'draw', function( ctx ) {
    "use strict";
    ctx.moveTo( this.p1.x, this.p1.y );
    ctx.lineTo( this.p2.x, this.p2.y );
      
});
Line.method( 'relAngles', function( other ) {
    "use strict";
    return Math.abs( this.theta - other.theta );
} );
Line.method( 'toString', function() {
    "use strict";
    return this.p1.toString() + ', ' + this.p2.toString();
} );
Line.method( 'intersectSegments', function( other ) {
    "use strict";
    var isect = this.intersectLines( other );
    if ( this.pointInBounds( isect )
       && other.pointInBounds( isect ) ) {
        return isect
    } else {
        return
    }
} );
Line.method( 'intersectLines', function( other ) {
    "use strict";
    var denom = this.m - other.m;
    var p = (other.b - this.b) / denom;
    var q = this.m * p + this.b;

    return new Point( Math.round(p), Math.round(q) );
} );
Line.method( 'pointInBounds', function( point ) {
    "use strict";

    return ( point.x >= Math.min( this.p1.x, this.p2.x)
           && point.x <= Math.max( this.p1.x, this.p2.x)
	   && point.y >= Math.min( this.p1.y, this.p2.y)
           && point.y <= Math.max( this.p1.y, this.p2.y)
           );
} );

// ----------------------------------------------------------------------
// helper
//
function canonical_angle( angle ) {
    "use strict";
    while ( angle > Math.PI / 2 ) {
	angle -= Math.PI;
    }
	
    while ( angle < Math.PI / -2 ) {
	angle += Math.PI;
    }
	
    return angle;
}

// ----------------------------------------------------------------------
function RandomLine(){
    "use strict";

    var centre = new RandomPoint();
    var theta = Math.random() * 2 * Math.PI
    var len = Math.round(Line.prototype.bounds.maxX * Math.random()  * Math.random());

    Line.call( this, centre, theta, len );
}
RandomLine.inheritsFrom( Line );

// ----------------------------------------------------------------------
// End of File
//
