module( 'Line.js', {
    setup : function() {
        setGlobals();
    }
});
test( 'Line()', function() {
    "use strict";
    var l = new Line( new Point( 100, 100 ), 0, 50);
    equals( l.toString(), '(75, 100), (125, 100)', 'l is a horizontal line' );
    equals( l.m, 0, 'l has right slope' );
    equals( l.theta, 0, 'l has right angle' );
    equals( l.b, 100, 'l has right y-intercept' );

    l = new Line( new Point( 100, 100 ), Math.PI/2, 50);
    equals( l.toString(), '(100, 75), (100, 125)', 'l is a vertical line' );
    equals( l.m, infinity() * infinity(), 'l has right slope' );
    equals( l.theta, Math.PI/2, 'l has right angle' );
    ok( Math.abs(l.b) > infinity(), 'l has right y-intercept' );

    l = new Line( new Point( 100, 100 ), Math.PI/4, 50);
    equals( l.toString(), '(82, 82), (118, 118)', 'l is a diagonal line' );
    equals( l.m, 1, 'l has right slope' );
    equals( l.theta, Math.PI/4, 'l has right angle' );
    equals( l.b, 0, 'l has right y-intercept' );

    l = new Line( new Point( 100, 100 ), Math.PI/6, 50);
    equals( l.toString(), '(78, 88), (122, 112)', 'l is a 30 degree line' );
    equals( l.m, 30.0/55.0, 'l has right slope' );
    ok( Math.abs( l.theta - Math.atan2( .5, .866025)) < epsilon(),
        'l has right angle' );
    ok( l.b - 2500.0/55 < epsilon(), 'l has right y-intercept' );

});

test( 'distance()', function() {
    "use strict";

    var l = new Line( new Point( 100, 100 ), 0, 50);

    var p = new Point( 100, 125 ); // right angles
    var dist = l.distance( p );
    equals( dist, 25, 'horiz l is 25 units from ' + p.toString() );

    p = new Point( 120, 100 );  // on line  segment
    dist = l.distance( p );
    equals( dist, 0, 'horiz l contains the point ');

    p = new Point( 130, 100 );  // on line  segment
    dist = l.distance( p );
    equals( dist, 5, 'point is on line outside segment ');

    var l = new Line( new Point( 100, 100 ), Math.PI/2, 50);

    var p = new Point( 125, 100 ); // right angles
    var dist = l.distance( p );
    ok( Math.abs(dist - 25) < epsilon(), 'vert l is 25 units from ' + p.toString() );

    p = new Point( 100, 120 );  // on line  segment
    dist = l.distance( p );
    ok( Math.abs( dist ) < epsilon(), 'vert l contains the point ');

    p = new Point( 100, 130 );  // on line  segment
    dist = l.distance( p );
    equals( dist, 5, 'point is on line outside segment ');
    
});

test( 'intersct line extensions', function() {
    "use strict";

    var l1 = new Line ( new Point( 85, 85 ), Math.PI/4, 10 ),
    l2 = new Line ( new Point( 85, 115 ), Math.PI/-4, 10 );

    equal( l1.intersectLines( l2 ).toString(), '(100, 100)',
           'extend lines to find intersection' );
    
    var p = new Point( 100, 100 );
    var l1 = new Line( p, Math.PI / 6, 50 ),
    l2 = new Line( p, Math.PI / 4, 50 ),
    l3 = new Line( new Point(100, 106), Math.PI / 6, 50 );

    equals( l1.intersectSegments( l2 ).toString(), '(100, 100)',
            'detects intersection at midpoint' );
    equals( l1.intersectSegments( l3 ), undefined,
            'detects parallel lines' );
    equals( l3.intersectSegments( l2 ).toString(), '(113, 113)',
            'detects random intersection' );
});

function approx( num ) {
    "use strict";

    return Math.round( num * infinity()) / infinity();
}
test( 'relative angles', function() {
    "use strict";
    // test at 15 degree increments
    //
    var std = [ 0, Math.PI/12, 2*Math.PI/12, 3*Math.PI/12, 4*Math.PI/12, 
                5*Math.PI/12, 6*Math.PI/12, -5*Math.PI/12, -4*Math.PI/12, 
                -3*Math.PI/12, -2*Math.PI/12, -1*Math.PI/12
              ];
    var line = new Line ( new Point( 100, 100 ), 0, 10 );
    for ( var i = 1; i < 24; i++ ) {
        var other = new Line ( new Point( 100, 100 ), i * Math.PI/12, 10 );
        var theta = i * Math.PI/12,
        canonical = canonical_angle( theta );
        equal( approx(canonical), approx(std[i%12]),
               'angle is right for ' + i + '/12 PI' );
        equals( line.relAngles( other ), Math.abs(canonical),
                'Detects angle of ' + i + '/12 PI' );
    }
    
});

test( 'bound test', function() {
    "use strict";

    var hl = new Line( new Point( 100, 100), 0, 10 ),
    vl = new Line( new Point( 100, 100), Math.PI / 2, 10 ),
    dl = new Line( new Point( 100, 100), Math.PI/4, 14 );
    
    // horizontal line test
    ok( ! hl.pointInBounds( new Point(100, 99)),
        'pixel above horizontal line is out of bounds' );
    ok(  hl.pointInBounds( new Point(100, 100)),
         'pixel on horizontal line is in bounds' );
    ok( ! hl.pointInBounds( new Point(100, 101)),
        'pixel below horizontal line is out of bounds' );

    // vertical line test
    ok( ! vl.pointInBounds( new Point(99, 100)),
        'pixel  left of vertical line is out of bounds' );
    ok(  vl.pointInBounds( new Point(100, 100)),
         'pixel on vertical line is in bounds' );
    ok( ! vl.pointInBounds( new Point(101, 100)), 
        'pixel right of vertical line is out of bounds' );

    // diagonal line test
    ok( ! dl.pointInBounds( new Point(94, 100)),
        'pixel above of diagonal line upper bound is out of bounds' );
    ok(  dl.pointInBounds( new Point(95, 100)),
        'pixel on diagonal line upper bound is in bounds' );
    ok( dl.pointInBounds( new Point(96, 100)), 
        'pixel below  diagonal line upper bound isin bounds' );

    ok( dl.pointInBounds( new Point(104, 100)),
        'pixel above of diagonal line lower bound is in bounds' );
    ok(  dl.pointInBounds( new Point(105, 100)),
        'pixel on diagonal line lower bound is in bounds' );
    ok(! dl.pointInBounds( new Point(106, 100)), 
        'pixel below  diagonal line lower bound is out of bounds' );

    ok( ! dl.pointInBounds( new Point(100, 94)),
        'pixel left of diagonal line left bound is out of bounds' );
    ok(  dl.pointInBounds( new Point(100, 95)),
        'pixel on diagonal line left bound is in bounds' );
    ok( dl.pointInBounds( new Point(100, 96)), 
        'pixel right  diagonal line left bound isin bounds' );

    ok( dl.pointInBounds( new Point(100, 104)),
        'pixel left of diagonal line right bound is in bounds' );
    ok(  dl.pointInBounds( new Point(100, 105)),
        'pixel on diagonal line right bound is in bounds' );
    ok(! dl.pointInBounds( new Point(100, 106)), 
        'pixel right  diagonal line right bound is out of bounds' );

});